#ifndef MSP_TEST_TESTFUNCTION_H_
#define MSP_TEST_TESTFUNCTION_H_

#include <string>
#include "exceptioncheck.h"

namespace Msp {
namespace Test {

class Test;

class Function
{
private:
	std::string description;
	ExceptionCheck *exc_check;

protected:
	Function(const std::string &);
public:
	virtual ~Function();

	const std::string &get_description() const { return description; }

	template<typename T>
	Function &expect_throw() { exc_check = new ExceptionTypeCheck<T>; return *this; }

	const ExceptionCheck *get_exception_check() const { return exc_check; }

	virtual void run(Test &) const = 0;
};

template<typename T>
class TypedFunction: public Function
{
private:
	typedef void (T::*FuncPtr)();

	FuncPtr func;

public:
	TypedFunction(FuncPtr f, const std::string & d):
		Function(d),
		func(f)
	{ }

	virtual void run(Test &t) const
	{
		(dynamic_cast<T &>(t).*func)();
	}
};

} // namespace Test
} // namespace Msp

#endif
