#include <msp/core/getopt.h>
#include <msp/io/print.h>
#include "runner.h"
#include "test.h"

using namespace std;

namespace Msp {
namespace Test {

Runner::Runner(int argc, char **argv):
	verbose(false),
	show_list(false)
{
	GetOpt getopt;
	getopt.add_option('v', "verbose", verbose,   GetOpt::NO_ARG).set_help("Show what is being done");
	getopt.add_option('l', "list",    show_list, GetOpt::NO_ARG).set_help("Print a list of available test cases");
	getopt(argc, argv);

	const vector<string> &args = getopt.get_args();
	tests.assign(args.begin(), args.end());
}

int Runner::main()
{
	if(show_list)
	{
		const list<Test::Factory *> &factories = Test::get_factories();
		for(list<Test::Factory *>::const_iterator i=factories.begin(); i!=factories.end(); ++i)
			IO::print("%s\n", (*i)->get_name());
	}
	else if(tests.empty())
		Test::run_all(verbose);
	else
	{
		for(list<string>::const_iterator i=tests.begin(); i!=tests.end(); ++i)
			Test::run_single(*i, verbose);
	}

	return 0;
}

} // namespace Test
} // namespace Msp
